# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import os
from django.utils.text import slugify
from functools import partial
from django.forms import TypedChoiceField
import re
from collections import namedtuple, OrderedDict
from datetime import datetime, timedelta, date, time
from decimal import Decimal
from itertools import chain, groupby
import json
import logging
from threading import RLock
from uuid import UUID
from django.conf import settings
from django.contrib.staticfiles.finders import get_finders
from django.core.files.storage import default_storage
from django.core.urlresolvers import NoReverseMatch, reverse
from django.utils.html import strip_tags
from django.utils.translation import ugettext as _
from django.core.exceptions import ValidationError
from django.core.validators import (validate_ipv46_address, URLValidator,
                                    validate_email)
from django.db.models import QuerySet, Model
from django import forms
from django.utils.encoding import force_text
from django.utils.encoding import python_2_unicode_compatible
from django.utils.module_loading import import_string
from django.utils.six import string_types, integer_types
from .validators import validate_setting_name, validate_default
from .validators import validate_formish
from django.core.serializers.json import DjangoJSONEncoder

try:
    forms.fields.CallableChoiceIterator
    CALLABLE_CHOICES = True
except AttributeError:
    # Django 1.7 doesn't have a way of lazily evaluating a set of choices.
    CALLABLE_CHOICES = False


logger = logging.getLogger(__name__)


class JSONEncoder(DjangoJSONEncoder):
    def default(self, o):
        if callable(o):
            o = o()
        if isinstance(o, datetime):
            # We don't use `T` as the sep, because Django doesn't include it
            # in it's DATETIME_INPUT_FORMATS
            r = o.isoformat(sep=str(' '))
            # We don't strip microseconds, because loss of precision is dumb
            # and I don't actually *care* about ECMA-262 or whatever.

            # We remove the timezone stuff because it ain't in
            # the DATETIME_INPUT_FORMATS
            if r.endswith('+00:00'):
                r = r[:-6]
            return r
        elif isinstance(o, timedelta):
            return force_text(o.total_seconds())
        elif isinstance(o, UUID):
            return force_text(o)
        elif isinstance(o, QuerySet):  # MultipleModelChoice
            return tuple(force_text(x.pk) for x in o)
        elif isinstance(o, Model):  # ModelChoice
            return force_text(o.pk)
        else:
            return super(JSONEncoder, self).default(o)  # pragma: no cover


class RegistryError(KeyError):
    pass

class AlreadyRegistered(RegistryError):
    pass

class NotRegistered(RegistryError):
    pass


Unregistered = namedtuple('Unregistered', 'setting_name form_class default')


@python_2_unicode_compatible
class FormRegistry(object):
    __slots__ = ('_registry', '_defaults', '_name', '_lock')

    def __init__(self, name=None):
        self._registry = {}
        self._defaults = {}
        self._name = name or 'default'
        self._lock = RLock()

    def __str__(self):
        return ', '.join(self._registry.keys())

    def __repr__(self):
        return '<%(mod)s.%(cls)s "%(name)s" [%(settings)s]>' % {
            'mod': self.__module__, 'cls': self.__class__.__name__,
            'name': self._name, 'settings': force_text(self),
        }

    def __len__(self):
        return len(self._registry)

    def ready(self, sender, instance, model):
        project_setting = getattr(settings, 'STAGESETTINGS', {})

        for setting_name, config in project_setting.items():
            default = None
            first_param_is_dictish = False
            # if the only parameter is a dictionary, make it a form.
            try:
                validate_default(config)
                form = generate_form(config)
                default = config
            except ValidationError:
                # if the first parameter is a dictionary, generate a form
                try:
                    validate_default(config[0])
                    form = generate_form(config[0])
                    default = config[0]
                    first_param_is_dictish = True
                # if it's not a dictionary, it's a path to a Form class
                except ValidationError:
                    form = import_string(config[0])
                    first_param_is_dictish = False
            # if a second parameter is given, it is always the defaults.
            try:
                default = config[1]
            except IndexError:
                logger.info("No default found in the second parameter "
                            "for %s" % setting_name)
            except KeyError:
                logger.info("%s config is a dictionary, assuming it represents "
                            "both the form and default values" % setting_name)
            if first_param_is_dictish and default is not None:
                duped = config[0].copy()
                # don't call .update(), instead only copy in the ones which
                # existed in the form ...
                for k, v in default.items():
                    if k in duped:
                        duped[k] = v
                default = duped
            self.register(key=setting_name, form_class=form, default=default)
        return None
        # db_for_rw = model.objects.using(self._name)
        # wanted = set(self.keys())
        # existing = set(db_for_rw.filter(key__in=wanted)
        #                .values_list('key', flat=True))
        # missing = wanted - existing
        # models = [model(key=setting_name,
        #                 raw_value=self.get_default(key=setting_name))
        #           for setting_name in missing]
        # return models

    def register(self, key, form_class, default=None):
        validate_setting_name(key)
        validate_formish(form_class)
        if default is not None:
            validate_default(default)
        with self._lock:
            if key in self._registry:
                raise AlreadyRegistered('The setting "%s" is already registered' % key)
            self._registry[key] = form_class
            self._defaults[key] = default or {}
            return True
    add = register
    __setitem__ = register

    def unregister(self, key):
        validate_setting_name(key)
        with self._lock:
            if key not in self._registry:
                raise NotRegistered('The setting "%s" is not registered' % key)
            existing_form = self._registry.pop(key)
            existing_default = self._defaults.pop(key)
            return Unregistered(setting_name=key, form_class=existing_form,
                                default=existing_default)
    remove = unregister
    pop = unregister
    __delitem__ = unregister

    def __getitem__(self, item):
        return self._registry[item]

    def __nonzero__(self):
        return len(self) > 0
    __bool__ = __nonzero__

    def keys(self):
        return self._registry.keys()

    def values(self):
        return self._registry.values()

    def _get_default(self, key):
        validate_setting_name(key)
        return self._defaults[key]

    def get_default(self, key):
        return self.serialize(self._get_default(key=key))

    def serialize(self, data):
        return json.dumps(data, cls=JSONEncoder)

    def deserialize(self, data):
        return json.loads(data)

registry = FormRegistry(name='default')


def _get_files_in_static_storage(only_matching=None):
    finders = get_finders()
    finders_results = (finder.list(ignore_patterns=None) for finder in finders)
    found_files = chain.from_iterable(finders_results)
    filenames_only = (filename for filename, storage in found_files)
    if only_matching is not None:
        # Apply a regex search over the given filenames to choose only
        # matching elements.
        filenames_only = (fname for fname in filenames_only
                          if re.search(only_matching, fname))
    return filenames_only


def make_storage_choices(found_files):
    def first_part_of_path(x):
        """
        Ensure that items without a directory part are grouped together before
        being passed to groupby
        """
        split_by_path_sep = x.split('/')
        leftmost = split_by_path_sep[0]
        if leftmost == x:
            return _('None')
        return leftmost

    filenames_only = sorted(found_files, key=first_part_of_path)

    def tuple_to_choices(groupname, flat_tuple):
        """
        Given a tuple of:
            ('path/subpath/filename',
             'path/another_path/filename2')
        Yield something like:
            (('path/subpath/filename', 'subpath/filename'),
             ('path/another_path/filename2', 'another_path/filename2'))
        where `path` got replaced in the display value based on the
        given `groupname`
        """
        return tuple((v, v.replace(groupname, '', 1).lstrip('/'))
                     for v in flat_tuple)

    groups = groupby(filenames_only, first_part_of_path)
    evaluated_groups = tuple((groupname, tuple(groupitems))
                             for groupname, groupitems in groups)
    fixed = ((groupname, tuple_to_choices(groupname, groupitems))
             for groupname, groupitems in evaluated_groups
             if not groupname.startswith('.') and groupitems)
    # import pdb; pdb.set_trace()
    return fixed


def list_files_in_static(only_matching=None):
    return make_storage_choices(
        found_files=_get_files_in_static_storage(only_matching=only_matching))


class StaticFilesChoiceField(TypedChoiceField):
    def __init__(self, *args, **kwargs):
        if 'choices' not in kwargs:  # pragma: no cover
            kwargs['choices'] = list_files_in_static
        if CALLABLE_CHOICES is False:
            kwargs['choices'] = kwargs['choices']()
        kwargs['coerce'] = force_text
        super(StaticFilesChoiceField, self).__init__(*args, **kwargs)


class PartialStaticFilesChoiceField(StaticFilesChoiceField):
    def __init__(self, only_matching, *args, **kwargs):
        if 'choices' not in kwargs:  # pragma: no cover
            kwargs['choices'] = partial(list_files_in_static,
                                        only_matching=only_matching)
        super(PartialStaticFilesChoiceField, self).__init__(*args, **kwargs)


def _get_files_in_default_storage(directory=''):
    dirs, files = default_storage.listdir(directory)
    for fn in files:
        location = os.path.join(directory, fn)
        yield location
    for subdir in dirs:
        if directory:
            subdir = os.path.join(directory, subdir)
        for fn in _get_files_in_default_storage(directory=subdir):
            yield fn


def list_files_in_default_storage(only_matching=None):
    filenames_only = _get_files_in_default_storage()
    if only_matching is not None:
        # Apply a regex search over the given filenames to choose only
        # matching elements.
        filenames_only = (fname for fname in filenames_only
                          if re.search(only_matching, fname))
    return make_storage_choices(found_files=filenames_only)


class DefaultStorageFilesChoiceField(TypedChoiceField):
    def __init__(self, *args, **kwargs):
        if 'choices' not in kwargs:  # pragma: no cover
            kwargs['choices'] = list_files_in_default_storage
        if CALLABLE_CHOICES is False:
            kwargs['choices'] = kwargs['choices']()
        kwargs['coerce'] = force_text
        super(DefaultStorageFilesChoiceField, self).__init__(*args, **kwargs)


class PartialDefaultStorageFilesChoiceField(DefaultStorageFilesChoiceField):
    def __init__(self, only_matching, *args, **kwargs):
        if 'choices' not in kwargs:  # pragma: no cover
            kwargs['choices'] = partial(list_files_in_default_storage,
                                        only_matching=only_matching)
        super(PartialDefaultStorageFilesChoiceField, self).__init__(*args, **kwargs)


def get_htmlfield(**kwargs):
    try:
        from django_bleach.forms import BleachField as HTMLField
    except ImportError:
        logger.warning("Using an HTML field without having `django-bleach` "
                       "installed; you should install it. Falling back to "
                       "using a CharField")
        HTMLField = forms.CharField

    def ckeditor_field(**kws):  # pragma: no cover
        try:
            reverse('ckeditor_upload')
        except NoReverseMatch:
            logging.info("Tried to use `django-ckeditor` without setting up "
                         "the URLconf requirements.", exc_info=1)
            return None
        # unlike most of the other fields,
        from ckeditor.widgets import CKEditorWidget
        kws.update(widget=CKEditorWidget)
        return HTMLField(**kws)

    def tinymce_field(**kws):  # pragma: no cover
        from tinymce.widgets import TinyMCE
        kws.update(widget=TinyMCE)
        return HTMLField(**kws)

    def django_markdown_field(**kws):  # pragma: no cover
        try:
            reverse('django_markdown_preview')
        except NoReverseMatch:
            logging.info("Tried to use `django_markdown` without setting up "
                         "the URLconf requirements", exc_info=1)
            return None
        from django_markdown.widgets import MarkdownWidget
        kws.update(widget=MarkdownWidget)
        return HTMLField(**kws)

    def pagedown_field(**kws):  # pragma: no cover
        from pagedown.widgets import PagedownWidget
        kws.update(widget=PagedownWidget)
        return HTMLField(**kws)

    def epiceditor_field(**kws):  # pragma: no cover
        from epiceditor.widgets import EpicEditorWidget
        kws.update(widget=EpicEditorWidget)
        return HTMLField(**kws)

    known = OrderedDict([
        ('ckeditor', ckeditor_field),
        ('tinymce', tinymce_field),
        ('django_markdown', django_markdown_field),
        ('pagedown', pagedown_field),
        ('epiceditor', epiceditor_field),
    ])
    for appname, field_function in known.items():
        if appname not in settings.INSTALLED_APPS:  # pragma: no cover
            continue
        field = field_function(**kwargs)
        if field is None:
            # Some of the widgets need urls configured. If they haven't been,
            # they're unusable so try the next one.
            continue
        return field
    # Bleach or CharField.
    return HTMLField(**kwargs)


def _select_field(v):
    if callable(v):
        v = v()
    if v is None:
        return forms.NullBooleanField(initial=v)
    elif isinstance(v, datetime):
        return forms.DateTimeField(initial=v)
    elif isinstance(v, date):
        return forms.DateField(initial=v)
    elif isinstance(v, time):
        return forms.TimeField(initial=v)
    elif isinstance(v, timedelta):
        return forms.DurationField(initial=v)
    elif isinstance(v, Decimal):
        return forms.DecimalField(initial=v)
    elif isinstance(v, float):
        return forms.FloatField(initial=v)
    elif isinstance(v, bool):
        return forms.BooleanField(initial=v, required=False)
    elif isinstance(v, integer_types):
        return forms.IntegerField(initial=v)
    elif isinstance(v, UUID):
        try:
            return forms.UUIDField(initial=v)
        except AttributeError:
            logger.error("You're on an older version of Django which doesn't "
                         "have UUIDField, so a charfield is being used "
                         "instead", exc_info=1)
            return forms.CharField(initial=str(v))
    elif isinstance(v, (list, tuple)):
        choices = tuple((k, k) for k in v)
        return forms.MultipleChoiceField(choices=choices)
    elif isinstance(v, OrderedDict):
        choices = tuple((k, subv) for k, subv in v.items())
        return forms.ChoiceField(choices=choices)
    elif isinstance(v, dict):
        choices = sorted((k, subv) for k, subv in v.items())
        return forms.ChoiceField(choices=choices)
    elif isinstance(v, (set, frozenset)):
        choices = sorted((k, k) for k in v)
        return forms.ChoiceField(choices=choices)
    elif isinstance(v, Model):
        kws = {'queryset': v.__class__.objects.all()}
        if hasattr(v, 'pk') and v.pk is not None:
            kws.update(initial=v.pk)
        return forms.ModelChoiceField(**kws)
    elif isinstance(v, QuerySet):
        return forms.ModelMultipleChoiceField(queryset=v)
    elif isinstance(v, re._pattern_type):
        return forms.RegexField(regex=v)
    elif isinstance(v, string_types):
        try:
            validate_ipv46_address(v)
            return forms.GenericIPAddressField(initial=v)
        except ValidationError:
            pass
        try:
            URLValidator()(v)
            return forms.URLField(initial=v)
        except ValidationError:
            pass
        try:
            validate_email(v)
            return forms.EmailField(initial=v)
        except ValidationError:
            pass

        if v in (settings.STATIC_URL, settings.STATICFILES_STORAGE):
            return StaticFilesChoiceField()

        if v in (settings.MEDIA_URL, settings.DEFAULT_FILE_STORAGE):
            return DefaultStorageFilesChoiceField()

        # filtered by a raw string representing a regex.
        starts_with_static_url = v.startswith(settings.STATIC_URL)
        if starts_with_static_url:
            v = v[len(settings.STATIC_URL):]
            return PartialStaticFilesChoiceField(only_matching=v)

        # filtered by a raw string representing a regex.
        starts_with_media_url = v.startswith(settings.MEDIA_URL)
        if starts_with_media_url:
            v = v[len(settings.MEDIA_URL):]
            return PartialDefaultStorageFilesChoiceField(only_matching=v)

        # filtered by a raw string representing a tethered regex.
        starts_with_tethered_static_url = v.startswith('^%s' % settings.STATIC_URL)  # noqa
        if starts_with_tethered_static_url:
            # removes ^/static/ or whatever and instead tethers to the
            # beginning of the next part.
            # so ^/static/admin/.+$ should become ^/admin/.+$
            v = '^%s' % v[len('^%s' % settings.STATIC_URL):]
            return PartialStaticFilesChoiceField(only_matching=v)

        # filtered by a raw string representing a tethered regex.
        starts_with_tethered_media_url = v.startswith('^%s' % settings.MEDIA_URL)  # noqa
        if starts_with_tethered_media_url:
            # removes ^/media/ or whatever and instead tethers to the
            # beginning of the next part.
            # so ^/media/admin/.+$ should become ^/admin/.+$
            v = '^%s' % v[len('^%s' % settings.MEDIA_URL):]
            return PartialDefaultStorageFilesChoiceField(only_matching=v)

        if ' ' not in v and '-' in v and slugify(v) == v:
            return forms.SlugField(initial=v)

        if strip_tags(v) != v:
            return get_htmlfield(initial=v)

        return forms.CharField(initial=v)


def generate_form(dictionary):
    form_fields = OrderedDict()
    for k, v in sorted(dictionary.items()):
        if callable(v):
            v = v()
        newfield = _select_field(v)
        if newfield is not None:
            form_fields[k] = newfield
    if len(form_fields) != len(dictionary):
        raise ValidationError("Could not generate all fields for form")
    return type(str('DictionaryGeneratedForm'), (forms.Form,), form_fields)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.forms import fields
from django.contrib.admin import widgets as admin_widgets
from django.core.cache.backends.base import MEMCACHE_MAX_KEY_LENGTH
from django.core.exceptions import ValidationError
from django.db.models import BLANK_CHOICE_DASH
from django.forms import Form, Select
from django.forms.fields import CharField
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _
from .models import RuntimeSetting
from stagesetting import widgets
from stagesetting.utils import registry
import warnings


class CreateSettingForm(Form):
    key = CharField(min_length=1, max_length=MEMCACHE_MAX_KEY_LENGTH,
                    label=_("Name"))

    def __init__(self, *args, **kwargs):
        super(CreateSettingForm, self).__init__(*args, **kwargs)
        existing = frozenset(RuntimeSetting.objects.values_list('key', flat=True))
        possible = frozenset(registry.keys())
        creatable = possible - existing
        final = list((x, x) for x in creatable)
        self.fields['key'].widget = Select(choices=BLANK_CHOICE_DASH + final)


    def clean_key(self):
        key = self.cleaned_data['key']
        try:
            exists = RuntimeSetting.objects.key_exists(key=key)
        except RuntimeSetting.DoesNotExist as e:
            raise ValidationError(force_text(e))
        if exists:
            raise ValidationError("Setting already exists")
        return key

    def save(self):
        key = self.cleaned_data['key']
        value = registry.get_default(key=key)
        obj = RuntimeSetting(key=key, raw_value=value)
        obj.full_clean()
        obj.save()
        return obj



ADMINFORMFIELD_FOR_FORMFIELD_DEFAULTS = {
    fields.DateTimeField: {
    #     'form_class': fields.SplitDateTimeField,
        'widget': admin_widgets.AdminSplitDateTime
    },
    # fields.SplitDateTimeField: {'widget': admin_widgets.AdminSplitDateTime},
    fields.DateField: {'widget': admin_widgets.AdminDateWidget},
    fields.TimeField: {'widget': admin_widgets.AdminTimeWidget},
    fields.URLField: {'widget': admin_widgets.AdminURLFieldWidget},
    fields.IntegerField: {'widget': widgets.AdminIntegerFieldReplacement},
    fields.CharField: {'widget': admin_widgets.AdminTextInputWidget},
    fields.ImageField: {'widget': admin_widgets.AdminFileWidget},
    fields.FileField: {'widget': admin_widgets.AdminFileWidget},
    fields.EmailField: {'widget': admin_widgets.AdminEmailInputWidget},
    # ModelMultipleChoiceField: {'widget': partial(widgets.FilteredSelectMultiple, verbose_name='hello', is_stacked=False)}
}


class AdminFieldForm(object):
    def __init__(self, *args, **kwargs):
        super(AdminFieldForm, self).__init__(*args, **kwargs)
        for field_name, field in self.fields.items():
            if field.__class__ in ADMINFORMFIELD_FOR_FORMFIELD_DEFAULTS:
                custom = ADMINFORMFIELD_FOR_FORMFIELD_DEFAULTS[field.__class__]
                old_attrs = field.widget.attrs.copy()
                if field.widget == field.__class__.widget:
                    field.widget = custom['widget'](attrs=old_attrs)
            if field.__class__ == fields.SplitDateTimeField:
                warnings.warn("Don't use SplitDateTimeField it's a multiwidget "
                              "and they're kind of a pain to split.", RuntimeWarning)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from rest_framework.fields import Field, DictField
from rest_framework.serializers import ModelSerializer
from rest_framework.viewsets import ModelViewSet
from stagesetting.models import RuntimeSetting
from stagesetting.utils import registry


class RawValueConversionField(DictField):
    def to_representation(self, value):
        if not isinstance(value, dict):
            return registry.deserialize(value)
        return super(RawValueConversionField, self).to_representation(value=value)


class RuntimeSettingSerializer(ModelSerializer):
    value = RawValueConversionField(source='raw_value')

    def validate(self, attrs):
        model = self.Meta.model()
        model.key = attrs['key']
        model.value = attrs['raw_value']
        return {'key': model.key, 'raw_value': model.raw_value}

    class Meta:
        model = RuntimeSetting
        fields = ('key', 'value')


class SettingsViewSet(ModelViewSet):
    queryset = RuntimeSetting.objects.all()
    serializer_class = RuntimeSettingSerializer



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import logging
from django.contrib import messages
from django.contrib.admin.models import LogEntry, CHANGE
from django.contrib.admin.options import IS_POPUP_VAR
from django.contrib.admin.options import get_content_type_for_model
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse, reverse_lazy
from django.db import transaction
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _, string_concat
from django.views.generic import FormView
from django.views.generic import ListView
from django.views.generic import DeleteView
from .models import RuntimeSetting
from .forms import CreateSettingForm, AdminFieldForm
from .utils import registry


logger = logging.getLogger(__name__)


def request_passes_test(request, obj=None):
    ok = (request.user.is_authenticated() and
          request.user.is_active and request.user.is_staff)
    if not ok:
        raise PermissionDenied("User is anonymous, inactive or is not staff")
    return ok


class CreateSetting(FormView):
    form_class = CreateSettingForm
    template_name = 'stagesetting/create.html'
    success_url_name = 'stagesetting_update'
    admin = False

    def get_context_data(self, **kwargs):
        ctx = super(CreateSetting, self).get_context_data(**kwargs)
        ctx.update(
            opts=RuntimeSetting._meta,
            change=False,
            is_popup=(IS_POPUP_VAR in self.request.POST or
                      IS_POPUP_VAR in self.request.GET),
            save_as=False,
            has_delete_permission=self.request.user.has_perm('stagesetting.has_delete_permission'),
            has_add_permission=self.request.user.has_perm('stagesetting.has_add_permission'),
            show_save_and_continue=False,
            has_change_permission=self.request.user.has_perm('stagesetting.has_change_permission'),
            title=_("Add new setting"),
            add=True,
        )
        return ctx

    def assert_has_permission(self, request):
        return request_passes_test(request=request)

    def get(self, request, *args, **kwargs):
        self.assert_has_permission(request=request)
        return super(CreateSetting, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.assert_has_permission(request=request)
        return super(CreateSetting, self).post(request, *args, **kwargs)

    def form_valid(self, form):
        with transaction.atomic():
            self.object = form.save()
        return super(CreateSetting, self).form_valid(form=form)

    def get_success_url(self):
        return reverse(self.success_url_name, args=(self.object.pk,))


class UpdateSetting(FormView):
    template_name = 'stagesetting/update.html'
    success_url = reverse_lazy('stagesetting_list')
    admin = None

    def get_context_data(self, **kwargs):
        ctx = super(UpdateSetting, self).get_context_data(**kwargs)
        ctx.update(
            opts=RuntimeSetting._meta,
            add=False,
            change=True,
            is_popup=(IS_POPUP_VAR in self.request.POST or
                      IS_POPUP_VAR in self.request.GET),
            save_as=False,
            has_delete_permission=self.request.user.has_perm('stagesetting.has_delete_permission'),
            has_add_permission=self.request.user.has_perm('stagesetting.has_add_permission'),
            show_save_and_continue=False,
            has_change_permission=self.request.user.has_perm('stagesetting.has_change_permission'),
            title=string_concat(_("Change "), self.object.key),
            original=self.object,
        )
        if self.admin:
            ctx.update(
                media=self.admin.media + ctx['form'].media,
            )
        return ctx

    def get_form_class(self):
        try:
            form = registry[self.object.key]
        except KeyError as e:
            msg = 'form_class missing for setting "%(key)s"' % {
                'key': self.object.key
            }
            logger.error(msg, exc_info=1)
            raise Http404(msg)
        if self.admin:
            cls_name = str('AdminFields%s' % form.__name__)
            parents = (AdminFieldForm, form)
            replaced_form = type(form)(cls_name, parents, {})
            return replaced_form
        return form

    def get_initial(self):
        return self.object.value

    def assert_has_permission(self, request, obj=None):
        return request_passes_test(request=request, obj=obj)

    def get_object(self, *args, **kwargs):
        return get_object_or_404(RuntimeSetting, pk=self.kwargs.get('pk'))

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.assert_has_permission(request=request, obj=self.object)
        return super(UpdateSetting, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.assert_has_permission(request=request, obj=self.object)
        return super(UpdateSetting, self).post(request, *args, **kwargs)

    def keys_changed(self, old, new):
        keys = []
        for key in new:
            if key in old and old[key] != new[key]:
                keys.append({'key': key, 'old': old[key], 'new': new[key]})
            elif key not in old:
                keys.append({'key': key, 'old': None, 'new': new[key]})
        return keys

    def form_valid(self, form):
        with transaction.atomic():
            old_value = self.object.value
            changed_data = self.keys_changed(old_value, form.cleaned_data)
            self.object.raw_value = registry.serialize(data=form.cleaned_data)
            self.object.full_clean()
            self.object.save()
            LogEntry.objects.log_action(
                user_id=self.request.user.pk,
                content_type_id=get_content_type_for_model(self.object).pk,
                object_id=self.object.pk,
                object_repr=force_text(self.object),
                action_flag=CHANGE,
                change_message="Changed %(changed)s" % {
                    'old': force_text(old_value),
                    'new': force_text(self.object.raw_value),
                    'changed': force_text(registry.serialize(changed_data))
                }
            )
        msg_dict = {'name': force_text(self.object._meta.verbose_name),
                    'obj': force_text(self.object.key)}
        msg =  _('The %(name)s "%(obj)s" was changed successfully.') % msg_dict
        messages.success(self.request, msg)
        return super(UpdateSetting, self).form_valid(form=form)


class DeleteSetting(DeleteView):
    queryset = RuntimeSetting.objects.all()
    template_name = 'stagesetting/delete.html'
    admin = False
    success_url = reverse_lazy('stagesetting_list')

    def assert_has_permission(self, request, obj=None):
        return request_passes_test(request=request, obj=obj)

    def get_context_data(self, **kwargs):
        ctx = super(DeleteSetting, self).get_context_data(**kwargs)
        ctx.update(
            opts=RuntimeSetting._meta,
            original=self.object,
        )
        return ctx

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.assert_has_permission(request=request, obj=self.object)
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

    def delete(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.assert_has_permission(request=request, obj=self.object)
        success_url = self.get_success_url()
        self.object.delete()
        return HttpResponseRedirect(success_url)


class ListSettings(ListView):
    queryset = RuntimeSetting.objects.all()
    paginate_by = 20
    template_name = 'stagesetting/list.html'
    admin = False

    def assert_has_permission(self, request):
        return request_passes_test(request=request, obj=None)

    def get(self, request, *args, **kwargs):
        self.assert_has_permission(request=request)
        return super(ListSettings, self).get(request, *args, **kwargs)


create_view = CreateSetting.as_view()
delete_view = DeleteSetting.as_view()
update_view = UpdateSetting.as_view()
list_view = ListSettings.as_view()



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from collections import OrderedDict
import contextlib
import json
from django.core.urlresolvers import reverse
from django.forms import Form, IntegerField
import pytest
from pytest_django.lazy_django import skip_if_no_django
from stagesetting.models import RuntimeSetting
from stagesetting.utils import registry


@contextlib.contextmanager
def form(key):
    class ListPerPageForm(Form):
        count = IntegerField(initial=25, min_value=1, max_value=99)
    registry.register(key, ListPerPageForm, {'amdefault': None})
    try:
        yield ListPerPageForm
    finally:
        registry.unregister(key)


@pytest.fixture()
def api_client():
    """A Django test client instance."""
    skip_if_no_django()
    from rest_framework.test import APIClient
    return APIClient()


@pytest.mark.django_db
def test_api_getlist(api_client):
    test_value = {'testing': 1, 'count': 2}
    setting = RuntimeSetting(key="TEST")
    with form('TEST'):
        setting.value = test_value
    setting.save()
    url = reverse('runtimesetting-list')
    with form('TEST'):
        response = api_client.get(url)
    assert response.data == [
        OrderedDict(
            [('key', 'TEST'), 
             ('value', {'count': 2})]
        )
    ]



@pytest.mark.django_db
def test_api_get_single(api_client):
    test_value = {'testing': 1, 'count': 4}
    setting = RuntimeSetting(key="TEST2")
    with form('TEST2'):
        setting.value = test_value
    setting.save()
    url = reverse('runtimesetting-detail', args=(setting.pk,))
    with form('TEST2'):
        response = api_client.get(url)
    assert response.data == {
        'value': {'count': 4},
        'key': 'TEST2'
    }


@pytest.mark.django_db
def test_api_post_does_create(api_client):
    test_value = {
        'key': 'TEST3',
        'value': {'testing': 1, 'count': 4},
    }
    url = reverse('runtimesetting-list')
    with form('TEST3'):
        response = api_client.post(url, data=test_value, format='json')
    assert response.data == {
        'value': {'count': 4},
        'key': 'TEST3'
    }
    obj = RuntimeSetting.objects.get(key='TEST3')
    with form('TEST3'):
        assert obj.value == {'count': 4}


@pytest.mark.django_db
def test_api_put_does_update(api_client):
    initial_value = {
        'key': 'TEST4',
        'value': {'testing': 1, 'count': 1},
    }
    setting = RuntimeSetting(key="TEST4")
    with form('TEST4'):
        setting.value = initial_value
    setting.save()
    test_value = {
        'key': 'TEST4',
        'value': {'testing': 1, 'count': 25},
    }
    url = reverse('runtimesetting-detail', args=(setting.pk,))
    with form('TEST4'):
        response = api_client.put(url, data=test_value, format='json')
    assert response.data == {
        'value': {'count': 25},
        'key': 'TEST4'
    }
    obj = RuntimeSetting.objects.get(key='TEST4')
    with form('TEST4'):
        assert obj.value == {'count': 25}


@pytest.mark.django_db
def test_api_patch_does_update(api_client):
    initial_value = {
        'key': 'TEST5',
        'value': {'testing': 1, 'count': 1},
    }
    setting = RuntimeSetting(key="TEST5")
    with form('TEST5'):
        setting.value = initial_value
    setting.save()
    test_value = {
        'key': 'TEST5',
        'value': {'testing': 1, 'count': 25},
    }
    url = reverse('runtimesetting-detail', args=(setting.pk,))
    with form('TEST5'):
        response = api_client.patch(url, data=test_value, format='json')
    assert response.data == {
        'value': {'count': 25},
        'key': 'TEST5'
    }
    obj = RuntimeSetting.objects.get(key='TEST5')
    with form('TEST5'):
        assert obj.value == {'count': 25}



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from django.template import Library
from stagesetting.models import RuntimeSettingWrapper


register = Library()


@register.assignment_tag(takes_context=True)
def stagesetting(context):
    if 'STAGESETTING' in context:
        wrapper = context['STAGESETTING']
    elif 'request' in context and hasattr(context['request'], 'stagesetting'):
        wrapper = context['request'].stagesetting
    else:
        wrapper = RuntimeSettingWrapper()
    return wrapper



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
import json
from uuid import UUID, uuid4
from datetime import timedelta, datetime, date, time
from decimal import Decimal
from collections import OrderedDict
from django.conf import settings
from django.core.exceptions import ValidationError
from django.forms import fields, ModelChoiceField, ModelMultipleChoiceField
from django.utils.functional import empty
import re
import django
from django.contrib.auth import get_user_model
from django.test import TransactionTestCase
from django.test.utils import override_settings, patch_logger
from django.utils.timezone import utc
from django_bleach.forms import BleachField
import pytest
from stagesetting.models import RuntimeSetting
from stagesetting.utils import (JSONEncoder, FormRegistry, generate_form,
                                list_files_in_static, get_htmlfield,
                                list_files_in_default_storage,
                                StaticFilesChoiceField,
                                PartialStaticFilesChoiceField,
                                PartialDefaultStorageFilesChoiceField,
                                DefaultStorageFilesChoiceField)


@pytest.mark.django_db
def test_custom_json():
    user = get_user_model().objects.create()
    output = json.dumps({
        'uuid': UUID('98967ef2-a5a3-4c19-aefa-9bb8dc5fcbac'),
        'timedelta': timedelta(minutes=14),
        'datetime': datetime(2015, 8, 1, 16, 8, 51, 125068),
        'tzdatetime': datetime(2015, 8, 1, 16, 8, 51, 125068).replace(tzinfo=utc),
        'super': 1,
        'user': user,
        'user_qs': get_user_model().objects.all(),
    }, cls=JSONEncoder)
    assert '"timedelta": "840.0"' in output
    assert '"datetime": "2015-08-01 16:08:51.125068"' in output
    assert '"uuid": "98967ef2-a5a3-4c19-aefa-9bb8dc5fcbac"' in output
    assert '"tzdatetime": "2015-08-01 16:08:51.125068"' in output
    assert '"user_qs": ["1"]' in output
    assert '"user": "1"' in output


def formregistry_ready():
    fr = FormRegistry(name='default')
    newconfig = {
        'HELLO': ['test_app.forms.DatetimeForm', {'blip': 'blop'}],
        'HELLO2': ['test_app.forms.ListPerPageForm'],
    }
    with override_settings(STAGESETTINGS=newconfig):
        return fr.ready(sender=None, instance=None, model=RuntimeSetting)


@pytest.mark.django_db
def test_formregistry_ready_dict():
    fr = FormRegistry(name='default')
    newconfig = {
        'HELLO': ['test_app.forms.DatetimeForm', {'blip': 'blop'}],
        'HELLO2': {
            'int': 1,
            'email': 'a@b.com',
            'url': 'https://news.bbc.co.uk/',
        },
    }
    with override_settings(STAGESETTINGS=newconfig):
        with patch_logger('stagesetting.utils', 'info') as logger_calls:
            result = fr.ready(sender=None, instance=None, model=RuntimeSetting)
        # implicit = RuntimeSetting.objects.get(key='HELLO2')
        # assert fr.deserialize(implicit.raw_value) == {
        #     'email': 'a@b.com',
        #     'int': 1,
        #     'url': 'https://news.bbc.co.uk/'
        # }
        assert fr._get_default(key='HELLO2') == {
            "email": "a@b.com",
            "int": 1,
            "url": "https://news.bbc.co.uk/"
        }
    # assert len(result) == 2
    assert logger_calls == ['HELLO2 config is a dictionary, assuming it '
                            'represents both the form and default values']


@pytest.mark.django_db
def test_formregistry_ready_dict_with_different_defaults():
    fr = FormRegistry(name='default')
    newconfig = {
        'HELLO': ['test_app.forms.DatetimeForm', {'blip': 'blop'}],
        'HELLO2': [{
            'int': 1,
            'email': 'a@b.com',
            'url': 'https://news.bbc.co.uk/',
        }, {
            'int': 3,
            'email': 'a@b.com',
            'url': 'https://www.bbc.com/',
        }],
    }
    with override_settings(STAGESETTINGS=newconfig):
        result = fr.ready(sender=None, instance=None, model=RuntimeSetting)
        # implicit = RuntimeSetting.objects.get(key='HELLO2')
        # assert fr.deserialize(implicit.raw_value) == {
        #     'email': 'a@b.com',
        #     'int': 3,
        #     'url': 'https://www.bbc.com/'
        # }
        assert fr._get_default(key='HELLO2') == {
            "email": "a@b.com",
            "int": 3,
            "url": "https://www.bbc.com/"
        }
    # assert len(result) == 2


@pytest.mark.django_db
def test_formregistry_ready_dict_with_partial_defaults():
    fr = FormRegistry(name='default')
    newconfig = {
        'HELLO4': [{
            'int': 1,
            'email': 'a@b.com',
            'url': 'https://news.bbc.co.uk/',
        }, {
            'int': 12,
            'url': 'https://www.bbc.com/',
        }],
    }
    with override_settings(STAGESETTINGS=newconfig):
        result = fr.ready(sender=None, instance=None, model=RuntimeSetting)
        # implicit = RuntimeSetting.objects.get(key='HELLO4')
        # assert fr.deserialize(implicit.raw_value) == {
        #     'email': 'a@b.com',
        #     'int': 12,
        #     'url': 'https://www.bbc.com/'
        # }
        assert fr._get_default(key='HELLO4') == {
            "email": "a@b.com",
            "int": 12,
            "url": "https://www.bbc.com/"
        }
    # assert len(result) == 1


@pytest.mark.django_db
def test_formregistry_complex_dict_with_no_defaults():
    fr = FormRegistry(name='default')
    def user(): return get_user_model()(pk=2)
    def user_qs(): return get_user_model().objects.all()
    newconfig = {
        'COMPLEX_NO_DEFAULTS': {
            'model': user,
            'queryset': user_qs,
        },
    }
    with override_settings(STAGESETTINGS=newconfig):
        result = fr.ready(sender=None, instance=None, model=RuntimeSetting)
        # implicit = RuntimeSetting.objects.get(key='COMPLEX_NO_DEFAULTS')
        # assert fr.deserialize(implicit.raw_value) == {
        #     'model': '2',
        #     'queryset': []
        # }
        assert fr._get_default(key='COMPLEX_NO_DEFAULTS') == {
            'model': user,
            'queryset': user_qs,
        }
    # assert len(result) == 1


@pytest.mark.django_db
def test_formregistry_complex_dict_with_partial_defaults():
    fr = FormRegistry(name='default')
    def user(): return get_user_model()(pk=2)
    def user_qs(): return get_user_model().objects.all()
    newconfig = {
        'COMPLEX_NO_DEFAULTS': [{
            'model': user,
            'queryset': user_qs,
        }, {
            'model': 1,
        }]
    }
    with override_settings(STAGESETTINGS=newconfig):
        result = fr.ready(sender=None, instance=None, model=RuntimeSetting)
        # implicit = RuntimeSetting.objects.get(key='COMPLEX_NO_DEFAULTS')
        # assert fr.deserialize(implicit.raw_value) == {
        #     'model': 1,
        #     'queryset': []
        # }
        assert fr._get_default(key='COMPLEX_NO_DEFAULTS') == {
            'model': 1,
            'queryset': user_qs,
        }
    # assert len(result) == 1


@pytest.mark.django_db
def test_formregistry_complex_dict_with_different_partial_defaults():
    fr = FormRegistry(name='default')
    def user(): return get_user_model()(pk=2)
    def user_qs(): return get_user_model().objects.all()
    newconfig = {
        'COMPLEX_NO_DEFAULTS': [{
            'model': user,
            'queryset': user_qs,
        }, {
            'queryset': [11, 2],
        }]
    }
    with override_settings(STAGESETTINGS=newconfig):
        result = fr.ready(sender=None, instance=None, model=RuntimeSetting)
        # implicit = RuntimeSetting.objects.get(key='COMPLEX_NO_DEFAULTS')
        # assert fr.deserialize(implicit.raw_value) == {
        #     'model': '2',
        #     'queryset': [11, 2],
        # }
        assert fr._get_default(key='COMPLEX_NO_DEFAULTS') == {
            'model': user,
            'queryset': [11, 2],
        }
    # assert len(result) == 1


@pytest.mark.django_db
def test_formregistry_complex_dict_with_completely_different_defaults():
    fr = FormRegistry(name='default')
    def user(): return get_user_model()(pk=2)
    def user_qs(): return get_user_model().objects.all()
    newconfig = {
        'COMPLEX_NO_DEFAULTS': [{
            'model': user,
            'queryset': user_qs,
        }, {
            'model': 'ohgod',
            'wat': 'not-even-a-thing',  # this won't turn up ever, hopefully.
            'queryset': [11, 2],
        }]
    }
    with override_settings(STAGESETTINGS=newconfig):
        result = fr.ready(sender=None, instance=None, model=RuntimeSetting)
        # implicit = RuntimeSetting.objects.get(key='COMPLEX_NO_DEFAULTS')
        # assert fr.deserialize(implicit.raw_value) == {
        #     'model': 'ohgod',
        #     'queryset': [11, 2],
        # }
        assert fr._get_default(key='COMPLEX_NO_DEFAULTS') == {
            'model': 'ohgod',
            'queryset': [11, 2],
        }
    # assert len(result) == 1


class FormRegistryTestCase(TransactionTestCase):
    def test_ready(self):
        with self.assertNumQueries(0):
            result = formregistry_ready()
        self.assertIsNone(result)


@pytest.mark.django_db
def test_generate_form():
    def user():
        return get_user_model()()

    def user_qs():
        return get_user_model().objects.all()

    user = get_user_model().objects.create()

    data = {
        'int': 1,
        'email': 'a@b.com',
        'url': 'https://news.bbc.co.uk/',
        'model': user,
        'queryset': user_qs,
        'datetime': datetime.today(),
        'date': date.today(),
        'time': time(4, 23),
        'boolean': False,
        'nullboolean': None,
        'ip': '127.0.0.1',
        'slug': 'test-test',
        'text': 'char field',
        'decimal': Decimal('3.25'),
        'float': 2.3,
        'uuid': uuid4(),
        'list': ['a', 'b'],
        'set': {'a', 'b'},
        'regex': re.compile('test'),
        'ordereddict_choices': OrderedDict([
            ('a', '1'),
            ('2', 'two'),
        ]),
        'dict_choices': {
            'a': '1',
            '2': 'two',
        },
        'html': '<b>html!</b>',
        'static_url': settings.STATIC_URL,
        'static_storage': settings.STATICFILES_STORAGE,
        'partial_static_url': r'%sadmin/(.+)\.css' % settings.STATIC_URL,
        'partial_static_url2': r'^%sadmin/(.+)\.css|\.txt' % settings.STATIC_URL,
    }
    form_class = generate_form(data)
    uuid = uuid4()
    dt = datetime.today()
    daaate = dt.date()
    validate_data = {
        'int': 1,
        'email': 'a@b.com',
        'url': 'https://news.bbc.co.uk/',
        'model': str(user.pk),
        'queryset': [str(user.pk)],
        'datetime': dt.strftime('%Y-%m-%d %H:%M:%S'),
        'date': daaate.strftime('%Y-%m-%d'),
        'time': '4:23',
        'boolean': '',
        'nullboolean': '0',
        'ip': '127.0.0.1',
        'slug': 'test-test',
        'text': 'char field',
        'decimal': '3.25',
        'float': '2.3',
        'uuid': str(uuid),
        'list': ['a'],
        'set': 'b',
        'regex': 'test',
        'ordereddict_choices': 'a',
        'dict_choices': '2',
        'html': '<em>html!</em>',
        'static_url': 'admin/css/login.css',
        'static_storage': 'admin/css/login.css',
        'partial_static_url': 'admin/css/login.css',
        'partial_static_url2': 'admin/js/LICENSE-JQUERY.txt',
    }
    form = form_class(data=validate_data)
    valid = form.is_valid()
    assert form.errors == {}
    qs = form.cleaned_data.pop('queryset')
    uuid_result = form.cleaned_data.pop('uuid')
    assert form.cleaned_data == {
        'url': 'https://news.bbc.co.uk/',
        'text': 'char field',
        'nullboolean': None,
        'datetime': dt.replace(microsecond=0),
        'decimal': Decimal('3.25'),
        'boolean': False,
        'ip': '127.0.0.1',
        'float': 2.3,
        'email': 'a@b.com',
        'slug': 'test-test',
        'model': user,
        'int': 1,
        'list': ['a'],
        'time': time(4, 23),
        'set': 'b',
        'date': daaate,
        'regex': 'test',
        'dict_choices': '2',
        'ordereddict_choices': 'a',
        'html': '<em>html!</em>',
        'static_url': 'admin/css/login.css',
        'static_storage': 'admin/css/login.css',
        'partial_static_url': 'admin/css/login.css',
        'partial_static_url2': 'admin/js/LICENSE-JQUERY.txt',
    }
    assert valid is True


@pytest.mark.xfail(django.VERSION[:2] < (1, 8), reason="requires Django 1.8 to "
                                                       "use the duration field")
def test_generate_form_duration_field():
    data = {
        'timedelta': timedelta(days=2),
    }
    form_class = generate_form(data)
    validate_data = {
        'timedelta': '1',
    }
    form = form_class(data=validate_data)
    valid = form.is_valid()
    assert form.errors == {}
    assert form.cleaned_data == {
        'timedelta': timedelta(0, 1),
    }
    assert valid is True


def test_generate_form_cannot_figure_out_appropriate_type():
    data = {
        'unknown': empty,
    }
    with pytest.raises(ValidationError):
        form_class = generate_form(data)


def test_list_files_in_static():
    found = tuple(list_files_in_static())
    found2 = tuple(list_files_in_static())
    # make sure they're the same.
    assert found == found2
    assert len(found) == 1
    assert found[0][0] == 'admin'
    assert ('admin/js/LICENSE-JQUERY.txt', 'js/LICENSE-JQUERY.txt') in found[0][1]
    assert ('admin/css/changelists.css', 'css/changelists.css') in found[0][1]


def test_list_files_in_static_partial():
    found = tuple(list_files_in_static(only_matching='\.txt$'))
    found2 = tuple(list_files_in_static(only_matching='\.txt$'))
    # make sure they're the same.
    assert found == found2
    assert len(found) == 1
    assert found[0][0] == 'admin'
    assert found[0][1] == (('admin/js/LICENSE-JQUERY.txt', 'js/LICENSE-JQUERY.txt'),)


def test_list_files_in_default_storage():
    found = tuple(list_files_in_default_storage())
    found2 = tuple(list_files_in_default_storage())
    # make sure they're the same.
    assert found == found2
    assert found[0][0] == 'None'
    # Using negative indexes because py3k leaves __pycache__ folders behind
    # which break the numbering on Py2.
    assert found[-2][0] == 'static'
    assert found[-1][0] == 'templates'
    assert found[-2][1] == (('static/file_found_1.txt', 'file_found_1.txt'),
                           ('static/subdir/file_found_2.txt', 'subdir/file_found_2.txt'))
    assert found[-1][1] == (('templates/base.html', 'base.html'),
                            ('templates/example_usage.html', 'example_usage.html'))


def test_list_files_in_default_storage_partial():
    found = tuple(list_files_in_default_storage(only_matching='\.txt$'))
    found2 = tuple(list_files_in_default_storage(only_matching='\.txt$'))
    # make sure they're the same.
    assert found == found2
    assert len(found) == 1
    assert found[0][0] == 'static'
    assert found[0][1] == (('static/file_found_1.txt', 'file_found_1.txt'),
                           ('static/subdir/file_found_2.txt', 'subdir/file_found_2.txt'))


def test_get_htmlfield():
    lol = get_htmlfield(initial='woo')
    assert isinstance(lol, fields.CharField) is True
    assert isinstance(lol, BleachField) is True
    assert lol.initial == 'woo'


def test_static_files_choice_field():
    field = StaticFilesChoiceField()
    found = tuple(field.choices)
    assert found[0][0] == 'admin'
    assert ('admin/js/LICENSE-JQUERY.txt', 'js/LICENSE-JQUERY.txt') in found[0][1]
    assert ('admin/css/changelists.css', 'css/changelists.css') in found[0][1]


def test_partial_static_files_choice_field():
    field = PartialStaticFilesChoiceField(only_matching='\.txt$')
    found = tuple(field.choices)
    assert found[0][0] == 'admin'
    assert found[0][1] == (('admin/js/LICENSE-JQUERY.txt', 'js/LICENSE-JQUERY.txt'),)


def test_default_storage_files_choice_field():
    field = DefaultStorageFilesChoiceField()
    found = tuple(field.choices)
    assert found[-2][0] == 'static'
    assert found[-1][0] == 'templates'
    assert found[-2][1] == (('static/file_found_1.txt', 'file_found_1.txt'),
                           ('static/subdir/file_found_2.txt', 'subdir/file_found_2.txt'))
    assert found[-1][1] == (('templates/base.html', 'base.html'),
                            ('templates/example_usage.html', 'example_usage.html'))


def test_partial_default_storage_files_choice_field():
    field = PartialDefaultStorageFilesChoiceField(only_matching='\.txt$')
    found = tuple(field.choices)
    # Using negative indexes because py3k leaves __pycache__ folders behind
    # which break the numbering on Py2.
    assert found[0][0] == 'static'
    assert found[0][1] == (('static/file_found_1.txt', 'file_found_1.txt'),
                           ('static/subdir/file_found_2.txt', 'subdir/file_found_2.txt'))


def test_generate_form_none_becomes_nullbooleanfield():
    form = generate_form({'field': None})()
    assert isinstance(form.fields['field'], fields.NullBooleanField)


def test_generate_form_datetime_becomes_datetimefield():
    form = generate_form({'field': datetime.now()})()
    assert isinstance(form.fields['field'], fields.DateTimeField) is True


def test_generate_form_date_becomes_datefield():
    form = generate_form({'field': datetime.now().date()})()
    assert isinstance(form.fields['field'], fields.DateField) is True


def test_generate_form_time_becomes_timefield():
    form = generate_form({'field': time(4, 23)})()
    assert isinstance(form.fields['field'], fields.TimeField) is True


def test_generate_form_decimal_becomes_decimalfield():
    form = generate_form({'field': Decimal('1.0')})()
    assert isinstance(form.fields['field'], fields.DecimalField) is True


def test_generate_form_float_becomes_floatfield():
    form = generate_form({'field': 1.5})()
    assert isinstance(form.fields['field'], fields.FloatField) is True


def test_generate_form_bool_becomes_booleanfield():
    form = generate_form({'field': True})()
    assert isinstance(form.fields['field'], fields.BooleanField) is True


def test_generate_form_int_becomes_integerfield():
    form = generate_form({'field': 1})()
    assert isinstance(form.fields['field'], fields.IntegerField) is True


def test_generate_form_list_becomes_multiplechoicefield():
    form = generate_form({'field': ['a', 'b']})()
    assert isinstance(form.fields['field'], fields.MultipleChoiceField) is True


def test_generate_form_tuple_becomes_multiplechoicefield():
    form = generate_form({'field': ('a', 'b')})()
    assert isinstance(form.fields['field'], fields.MultipleChoiceField) is True


def test_generate_form_ordereddict_becomes_choicefield():
    form = generate_form({'field': OrderedDict([
        ['1', '2'],
        ['2', '4'],
    ])})()
    assert isinstance(form.fields['field'], fields.ChoiceField) is True


def test_generate_form_dict_becomes_choicefield():
    form = generate_form({'field': {
        'a': 'b',
        'c': 'd',
    }})()
    assert isinstance(form.fields['field'], fields.ChoiceField) is True


def test_generate_form_set_becomes_choicefield():
    form = generate_form({'field': {'a', 'b', 'c'}})()
    assert isinstance(form.fields['field'], fields.ChoiceField) is True


def test_generate_form_model_becomes_modelchoicefield():
    form = generate_form({'field': get_user_model()(pk=1)})()
    assert isinstance(form.fields['field'], ModelChoiceField) is True


def test_generate_form_queryset_becomes_modelmultiplechoicefield():
    form = generate_form({'field': get_user_model().objects.all()})()
    assert isinstance(form.fields['field'], ModelMultipleChoiceField) is True


def test_generate_form_regex_becomes_regexfield():
    form = generate_form({'field': re.compile('')})()
    assert isinstance(form.fields['field'], fields.RegexField) is True


def test_generate_form_ipish_string_becomes_genericipfield():
    form = generate_form({'field': '127.0.0.1'})()
    assert isinstance(form.fields['field'], fields.GenericIPAddressField) is True


def test_generate_form_urlish_string_becomes_urlfield():
    form = generate_form({'field': 'https://news.bbc.co.uk/'})()
    assert isinstance(form.fields['field'], fields.URLField) is True


def test_generate_form_emailish_string_becomes_emailfield():
    form = generate_form({'field': 'a@b.com'})()
    assert isinstance(form.fields['field'], fields.EmailField) is True


def test_generate_form_staticurl_becomes_staticfileschoicefield():
    form = generate_form({'field': settings.STATIC_URL})()
    assert isinstance(form.fields['field'], StaticFilesChoiceField) is True


def test_generate_form_staticstorage_becomes_staticfileschoicefield():
    form = generate_form({'field': settings.STATICFILES_STORAGE})()
    assert isinstance(form.fields['field'], StaticFilesChoiceField) is True



def test_generate_form_staticurl_startswith_becomes_staticfileschoicefield():
    form = generate_form({'field': '%s/test' % settings.STATIC_URL})()
    assert isinstance(form.fields['field'], PartialStaticFilesChoiceField) is True


def test_generate_form_mediaurl_becomes_staticfileschoicefield():
    form = generate_form({'field': settings.MEDIA_URL})()
    assert isinstance(form.fields['field'], DefaultStorageFilesChoiceField) is True


def test_generate_form_mediastorage_becomes_staticfileschoicefield():
    form = generate_form({'field': settings.DEFAULT_FILE_STORAGE})()
    assert isinstance(form.fields['field'], DefaultStorageFilesChoiceField) is True


def test_generate_form_mediaurl_startswith_becomes_staticfileschoicefield():
    form = generate_form({'field': '%s/test' % settings.MEDIA_URL})()
    assert isinstance(form.fields['field'], PartialDefaultStorageFilesChoiceField) is True


def test_generate_form_slugish_string_becomes_slugfield():
    form = generate_form({'field': 'a-b'})()
    assert isinstance(form.fields['field'], fields.SlugField) is True


def test_generate_form_slugish_string_has_invalid_slug_chars_slugfield():
    form = generate_form({'field': 'a-b!!!'})()
    assert isinstance(form.fields['field'], fields.SlugField) is False
    assert isinstance(form.fields['field'], fields.CharField) is True


def test_generate_form_slugish_string_has_invalid_slug_chars_slugfield2():
    form = generate_form({'field': 'a-b '})()
    assert isinstance(form.fields['field'], fields.SlugField) is False
    assert isinstance(form.fields['field'], fields.CharField) is True



# -*- coding: utf-8 -*-
__version_info__ = '0.4.0'
__version__ = '0.4.0'
version = '0.4.0'
def get_version(): return version  # pragma: no cover

default_app_config = 'stagesetting.apps.StageSettingAppConfig'



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import contextlib
from django.forms import Form, IntegerField
from django.test import TransactionTestCase, RequestFactory
from django.test.utils import patch_logger
from stagesetting.models import RuntimeSettingWrapper
from stagesetting.context_processors import runtime_settings
from stagesetting.middleware import ApplyRuntimeSettings
from stagesetting.utils import registry


@contextlib.contextmanager
def form(key):
    class ListPerPageForm(Form):
        count = IntegerField(initial=25, min_value=1, max_value=99)
    registry.register(key, ListPerPageForm, {'amdefault': None})
    yield ListPerPageForm
    registry.unregister(key)


def test_runtime_settings_without_middleware(rf):
    request = rf.get('/')
    output = runtime_settings(request=request)
    assert 'STAGESETTING' in output
    assert isinstance(output['STAGESETTING'], RuntimeSettingWrapper)
    return output


def test_runtime_settings_with_middleware(rf):
    request = rf.get('/')
    ApplyRuntimeSettings().process_view(request=request, view_func=None,
                                        view_args=(), view_kwargs={})
    output = runtime_settings(request=request)
    assert 'STAGESETTING' in output
    assert isinstance(output['STAGESETTING'], RuntimeSettingWrapper)
    assert output['STAGESETTING'] is request.stagesetting
    return output


def test_runtime_settings_with_middleware_doesnt_apply_twice(rf):
    request = rf.get('/')
    mw = ApplyRuntimeSettings()
    mw.process_view(request=request, view_func=None, view_args=(), view_kwargs={})
    with patch_logger('stagesetting.middleware', 'warning') as logger_calls:
        mw.process_view(request=request, view_func=None, view_args=(),
                        view_kwargs={})
        message = ['Another middleware already set `request.stagesetting`']
        assert logger_calls == message


class QueryTestCase(TransactionTestCase):
    def test_runtime_settings_without_middleware(self):
        with form('LIST_PER_PAGE'):
            with self.assertNumQueries(0):
                rf = RequestFactory()
                output = test_runtime_settings_without_middleware(rf=rf)
            with self.assertNumQueries(1):
                output['STAGESETTING'].LIST_PER_PAGE
            with self.assertNumQueries(0):
                output['STAGESETTING'].LIST_PER_PAGE

    def test_runtime_settings_with_middleware(self):
        with form('LIST_PER_PAGE'):
            with self.assertNumQueries(0):
                rf = RequestFactory()
                output = test_runtime_settings_with_middleware(rf=rf)
            with self.assertNumQueries(1):
                output['STAGESETTING'].LIST_PER_PAGE
            with self.assertNumQueries(0):
                output['STAGESETTING'].LIST_PER_PAGE



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.template import Template, Context
import pytest


def test_loading():
    assert Template("{% load stagesetting %}").render(Context()) == ''


@pytest.mark.django_db
def test_works():
    assert Template("{% load stagesetting %}{% stagesetting as LOL %}{{ LOL|length }}").render(Context()) == '2'



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.test import override_settings
from stagesetting.checks import check_setting
from stagesetting.checks import W001, E001, E003, E002, E004, E006, E007, E005, I001


def test_setting_not_set():
    result = check_setting(app_configs=None)
    assert len(result) == 1
    assert result[0].id == W001().id


def test_invalid_setting_name():
    settings = {
        'ghost': [],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 2
        assert result[0].id == E001().id


def test_invalid_value_type():
    settings = {
        'INVALID_TYPE': set(),
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 1
        assert result[0].id == E002().id


def test_invalid_value_length():
    settings = {
        'VALUE_LENGTH': [1, 2, 3],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 1
        assert result[0].id == E003().id


def test_invalid_type_for_value_argument_one():
    settings = {
        'NOT_A_STRING': [1],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 1
        assert result[0].id == E004().id

def test_invalid_dotted_path_for_value_argument_one():
    settings = {
        'NOT_A_STRING': ['this.should.NeverExist'],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 1
        assert result[0].id == E006().id


def test_invalid_form_object_for_value_argument_one():
    settings = {
        'NOT_A_STRING': ['django.VERSION'],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 1
        assert result[0].id == E007().id


def test_form_missing_full_defaults_but_wont_be_checked():
    settings = {
        'NOT_A_STRING': ['django.VERSION', {}],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 1
        assert result[0].id == E007().id


def test_form_missing_full_defaults():
    settings = {
        'NOT_A_STRING': ['django.contrib.auth.forms.AuthenticationForm', {
            'password': 'lolno',
        }],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 1
        assert result[0].id == I001(msg='').id


def test_invalid_datatype_for_value_argument_two_defauls():
    settings = {
        'NOT_A_STRING': ['django.VERSION', set()],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert len(result) == 2
        assert result[1].id == E005().id


def test_dictionary_value():
    settings = {
        'NOT_A_STRING': {
            'int': 1,
            'email': 'a@b.com',
            'url': 'https://news.bbc.co.uk/',
        },
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert result == []


def test_list_dictionary_first_value():
    settings = {
        'NOT_A_STRING': [{
            'int': 1,
            'email': 'a@b.com',
            'url': 'https://news.bbc.co.uk/',
        }],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert result == []


def test_list_dictionary_first_value_second_value():
    settings = {
        'NOT_A_STRING': [{
            'int': 1,
            'email': 'a@b.com',
            'url': 'https://news.bbc.co.uk/',
        }, {
            'int': 3
        }],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert result == []


def test_list_dictionary_first_value_second_value_is_list():
    settings = {
        'NOT_A_STRING': [{
            'int': 1,
            'email': 'a@b.com',
            'url': 'https://news.bbc.co.uk/',
        }, []],
    }
    with override_settings(STAGESETTINGS=settings):
        result = check_setting(app_configs=None)
        assert result[0].id == E005().id



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
import contextlib
import json
from django.contrib import admin
from django.core.urlresolvers import reverse
from django.forms import Form, IntegerField
import pytest
from stagesetting.forms import CreateSettingForm, AdminFieldForm
from stagesetting.models import RuntimeSetting
from stagesetting.utils import registry


@pytest.yield_fixture
def modeladmin():
    yield admin.site._registry[RuntimeSetting]


@pytest.yield_fixture
def add_url():
    url = reverse('admin:stagesetting_runtimesetting_add')
    yield url


@pytest.yield_fixture
def changelist_url():
    url = reverse('admin:stagesetting_runtimesetting_changelist')
    yield url


@pytest.yield_fixture
def change_url():
    x = RuntimeSetting.objects.create(key='GLORP',
                                      raw_value=json.dumps({'count': '13'}))
    url = reverse('admin:stagesetting_runtimesetting_change', args=(x.pk,))
    yield url

@pytest.yield_fixture
def delete_url():
    x = RuntimeSetting.objects.create(key='GLORP',
                                      raw_value=json.dumps({'count': '13'}))
    url = reverse('admin:stagesetting_runtimesetting_delete', args=(x.pk,))
    yield url


@contextlib.contextmanager
def form(key):
    class ThisForm(Form):
        count = IntegerField(initial=25, min_value=1, max_value=99)
    registry.register(key, ThisForm, {'amdefault': None})
    try:
        yield ThisForm
    finally:
        registry.unregister(key)


def test_history_link(modeladmin):
    result = modeladmin.history_link(obj=RuntimeSetting(pk=1))
    expected = ('<a href="/test_admin/stagesetting/runtimesetting/1/history/" '
                'class="historylink">History</a>')
    assert result == expected


@pytest.mark.django_db
def test_add_view_GET(admin_client, add_url):
    """
    :type admin_client: django.test.client.Client
    :type response: django.template.response.TemplateResponse
    """
    response = admin_client.get(add_url)
    assert response.status_code == 200
    assert response.context_data != {}
    assert response.context_data['add'] is True
    assert response.context_data['change'] is False
    assert response.context_data['save_as'] is False
    assert response.context_data['show_save_and_continue'] is False
    assert isinstance(response.context_data['form'], CreateSettingForm)


@pytest.mark.django_db
def test_add_view_POST(admin_client, add_url):
    """
    :type admin_client: django.test.client.Client
    :type response: django.template.response.TemplateResponse
    """
    with form('ADD_POST'):
        response = admin_client.post(add_url, {
            'key': 'ADD_POST',
        })
    assert response.status_code == 302
    x = RuntimeSetting.objects.get()
    url = reverse('admin:stagesetting_runtimesetting_change', args=(x.pk,))
    assert response.url.endswith(url)


@pytest.mark.django_db
def test_change_view_GET(admin_client, change_url):
    with form('GLORP'):
        response = admin_client.get(change_url)
    assert response.status_code == 200
    assert response.context_data != {}
    assert response.context_data['add'] is False
    assert response.context_data['change'] is True
    assert response.context_data['save_as'] is False
    assert response.context_data['show_save_and_continue'] is False
    assert isinstance(response.context_data['original'], RuntimeSetting)
    assert isinstance(response.context_data['form'], AdminFieldForm)


@pytest.mark.django_db
def test_change_view_POST(admin_client, change_url, changelist_url):
    with form('GLORP'):
        response = admin_client.post(change_url, {'count': '24'})
    assert response.status_code == 302
    assert response.url.endswith(changelist_url) is True


@pytest.mark.django_db
def test_delete_view_GET(admin_client, delete_url):
    response = admin_client.get(delete_url)
    assert response.status_code == 200
    assert response.context_data != {}
    assert isinstance(response.context_data['original'], RuntimeSetting)
    assert isinstance(response.context_data['object'], RuntimeSetting)
    assert isinstance(response.context_data['runtimesetting'], RuntimeSetting)


@pytest.mark.django_db
def test_delete_view_DELETE(admin_client, delete_url, changelist_url):
    with form('GLORP'):
        response = admin_client.delete(delete_url)
    assert response.status_code == 302
    assert response.url.endswith(changelist_url)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from functools import partial
from django.core.checks import Warning
from django.core.checks import Error
from django.core.checks import Info
from django.core.exceptions import ValidationError
from django.utils.six import string_types
from stagesetting.utils import generate_form

try:
    from django.utils.module_loading import import_string
except ImportError:  # pragma: no cover
    from django.utils.module_loading import import_by_path as import_string
from stagesetting.validators import validate_setting_name
from stagesetting.validators import validate_default
from stagesetting.validators import validate_formish

W001 = partial(Warning,
    msg="You don't have any settings defined",
    hint="Set STAGESETTINGS in your project settings module",
    id='stagesetting.W001',
)

I001 = partial(Info,
    id="stagesetting.I001",
)

E001 = partial(Error,
    msg="Invalid name found in config",
    hint="Setting names should be UPPER_CASE_WITH_UNDERSCORES",
    id="stagesetting.E001",
)

E002 = partial(Error,
    msg="The values for settings keys should be lists or tuples",
    id="stagesetting.E002",
)

E003 = partial(Error,
    msg="The tuple or list should be 1 or 2 indexes long",
    id="stagesetting.E003",
)

E004 = partial(Error,
    msg="The first value for the setting is invalid",
    hint="Provide the 'dotted.path.to.a.Form' as a string",
    id="stagesetting.E004",
)

E005 = partial(Error,
    msg="The first value for the setting is invalid",
    hint="The default data for a setting should be a dictionary of data which "
         "would be valid in the Form",
    id="stagesetting.E005",
)

E006 = partial(Error,
    msg="the form could not be imported",
    hint="Change the 'dotted.path.to.a.Form' to a valid import path",
    id="stagesetting.E006",
)

E007 = partial(Error,
    msg="The import path given does not appear to be a valid Form",
    hint="Use a standard form subclass",
    id="stagesetting.E007",
)


def validate_as_dict(key, value):
    """
    Returns `True` if it's a dict we can generate a form for,
    `False` if it's not, or an Error instance comes out of the form.
    """
    try:
        validate_default(value)
        form = generate_form(value)
    except ValidationError:
        return None
    try:
        validate_formish(form)
        return form
    except ValidationError:
        return None  # E007(obj=key)


def validate_params_of_list(key, value):
    """
    Assumes the `value` is a list ...
    """
    errors = []
    config_length = len(value)
    # has multiple values
    if not isinstance(value, (list, tuple)):
        try:
            wtf_is_it = type(value).__name__
        except AttributeError:  # pragma: no cover
            wtf_is_it = '???'
        return [E002(obj=key, hint="Got `%s` instead" % wtf_is_it)]
    if isinstance(value, (list, tuple)) and config_length not in (1, 2):
        return [E003(obj=key)]

    # first try to make a form
    form = validate_as_dict(key, value=value[0])
    ignore_defaults_set_check = True
    if form is None:
        ignore_defaults_set_check = False
        if not isinstance(value[0], string_types):
            return [E004(obj=key)]
        # then try and import a form
        try:
            form = import_string(value[0])
        except (ImportError, AttributeError) as exc:
            return [E006(obj=key)]
        # make sure our value feels like a form
        try:
            validate_formish(form)
        except ValidationError:
            errors.append(E007(obj=key))
            # we won't have `base_fields`, so don't check it ...
            ignore_defaults_set_check = True

    # make sure the second param feels like a dictionary
    if config_length == 2:
        try:
            validate_default(value[1])
        except ValidationError as exc:
            errors.append(E005(obj=key))
        else:
            # if the first part is not a dict, and the second part is,
            # report on what is missing.
            if ignore_defaults_set_check is False:
                fields_in_form = frozenset(form.base_fields.keys())
                fields_in_defaults = frozenset(value[1].keys())
                missing = fields_in_form - fields_in_defaults
                if missing:
                    errors.append(I001(
                        obj=key, msg='The following keys are not in '
                                     'the defaults: %s' % ', '.join(missing)))
    return errors


# noinspection PyUnusedLocal
def check_setting(app_configs, **kwargs):
    from django.conf import settings
    errors = []
    if not hasattr(settings, 'STAGESETTINGS'):
        errors.append(W001(obj='settings'))
    else:
        for key, value in settings.STAGESETTINGS.items():
            try:
                validate_setting_name(key)
            except ValidationError as exc:
                errors.append(E001(obj=key))

            only_param_is_dict = validate_as_dict(key, value)
            if only_param_is_dict is None:
                list_errors = validate_params_of_list(key, value)
                errors.extend(list_errors)
    return errors



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import logging
from .models import RuntimeSettingWrapper


logger = logging.getLogger(__name__)


class ApplyRuntimeSettings(object):
    __slots__ = ()

    def process_view(self, request, view_func, view_args, view_kwargs):
        if not hasattr(request, 'stagesetting'):
            request.stagesetting = RuntimeSettingWrapper()
        else:
            logger.warning("Another middleware already set `request.stagesetting`")  # noqa
        return None



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.contrib.admin.widgets import AdminIntegerFieldWidget


class AdminIntegerFieldReplacement(AdminIntegerFieldWidget):
    input_type = 'number'




# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.contrib import admin
from django.contrib.admin import ModelAdmin
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from .models import RuntimeSetting
from .views import CreateSetting
from .views import UpdateSetting
from .views import DeleteSetting


class RuntimeSettingAdmin(ModelAdmin):
    list_per_page = 500
    list_max_show_all = 500
    list_display = ['key', 'created', 'modified', 'has_changed', 'history_link']
    list_display_links = ['key']
    actions = None

    def history_link(self, obj):
        url = reverse('admin:stagesetting_runtimesetting_history', args=(obj.pk,))  # noqa
        return '<a href="%(url)s" class="historylink">%(label)s</a>' % {
            'url': url, 'label': _("History")
        }
    history_link.short_description = _(" ")
    history_link.allow_tags = True

    def add_view(self, request, **kwargs):
        return CreateSetting.as_view(
            admin=self, template_name='admin/stagesetting/add_form.html',
            success_url_name='admin:stagesetting_runtimesetting_change')(request=request)

    def change_view(self, request, object_id, **kwargs):
        return UpdateSetting.as_view(
            admin=self, template_name='admin/stagesetting/change_form.html',
            success_url=reverse('admin:stagesetting_runtimesetting_changelist'))(request=request, pk=object_id)

    def delete_view(self, request, object_id, **kwargs):
        return DeleteSetting.as_view(
            admin=self, template_name='admin/delete_confirmation.html',
            success_url=reverse('admin:stagesetting_runtimesetting_changelist'))(request=request, pk=object_id)
admin.site.register(RuntimeSetting, RuntimeSettingAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import sys
import os
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


HERE = os.path.abspath(os.path.dirname(__file__))


class PyTest(TestCommand):
    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


def make_readme(root_path):
    FILES = ('README.rst', 'LICENSE', 'CHANGELOG', 'CONTRIBUTORS')
    for filename in FILES:
        filepath = os.path.realpath(os.path.join(root_path, filename))
        if os.path.isfile(filepath):
            with open(filepath, mode='r') as f:
                yield f.read()


LONG_DESCRIPTION = "\r\n\r\n----\r\n\r\n".join(make_readme(HERE))


setup(
    name='django-stagesetting',
    version='0.4.0',
    packages=find_packages(exclude=['tests', 'test_app']),
    install_requires=(
        'Django>=1.7',
    ),
    tests_require=(
        'pytest>=2.6.4',
        'pytest-cov>=1.8.1',
        'pytest-django>=2.8.0',
        'pytest-remove-stale-bytecode>=1.0',
        'pytest-random>=0.2',
        'djangorestframework>=3.2',
        'django-bleach>=0.3.0',
    ),
    setup_requires=(
        "isort>=3.9.6",
    ),
    cmdclass={'test': PyTest},
    author='Keryn Knight',
    author_email='python-package@kerynknight.com',
    description="Dynamic runtime settings and configuration for Django sites",
    long_description=LONG_DESCRIPTION,
    keywords=['settings', 'django', 'live', 'dynamic', 'utility'],
    include_package_data=True,
    url='https://github.com/kezabelle/django-stagesetting',
    download_url='https://github.com/kezabelle/django-stagesetting/releases/tag/0.4.0',
    zip_safe=False,
    license="BSD License",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Environment :: Web Environment',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Framework :: Django',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
from django.conf.urls import url, include
from django.contrib import admin
from django.template.response import TemplateResponse
from rest_framework.routers import DefaultRouter
from stagesetting import urls as stagesetting_urls
from stagesetting.drf import SettingsViewSet


api = DefaultRouter(trailing_slash=True)
api.register(r'settings', SettingsViewSet)



def homepage(request):
    return TemplateResponse(request, template="example_usage.html", context={})


urlpatterns = [
   # url(r'docs/', include('rest_framework_swagger.urls')),
   url(r'api/', include(api.urls)),
   url(r'^$', homepage),
   url(r'^stagesetting/', include(stagesetting_urls)),
   url(r'^test_admin/', include(admin.site.urls)),
]



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals
import contextlib
import json
from django.contrib.auth import get_user_model
from django.forms import IntegerField, Form, ModelChoiceField, \
    ModelMultipleChoiceField
import pytest
from stagesetting.models import RuntimeSetting, RuntimeSettingWrapper
from stagesetting.utils import registry


@contextlib.contextmanager
def form(key):
    class ListPerPageForm(Form):
        count = IntegerField(initial=25, min_value=1, max_value=99)
    registry.register(key, ListPerPageForm, {'amdefault': None})
    try:
        yield ListPerPageForm
    finally:
        registry.unregister(key)


@contextlib.contextmanager
def userform(key):
    class ModelChoicesForm(Form):
        single_user = ModelChoiceField(queryset=get_user_model().objects.all())
        many_users = ModelMultipleChoiceField(queryset=get_user_model().objects.all())
        another = IntegerField(min_value=1, max_value=5)
    registry.register(key, ModelChoicesForm)
    try:
        yield ModelChoicesForm
    finally:
        registry.unregister(key)



def test_get_form_class():
    with form('TEST') as form_class:
        value = RuntimeSetting(key="TEST")
        assert value.get_form_class() == form_class
    with pytest.raises(KeyError):
        value.get_form_class()



def test_get_form():
    test_value = {'testing': 1, 'count': 2}
    with form('TEST') as form_class:
        value = RuntimeSetting(key="TEST", raw_value=json.dumps(test_value))
        form_instance = value.get_form()
        assert isinstance(form_instance, form_class)
        assert form_instance.is_bound is True
        assert form_instance.data == test_value


def test_value_deserializes_raw():
    test_value = {'testing': 1, 'count': 2}
    with form('TEST'):
        value = RuntimeSetting(key="TEST", raw_value=json.dumps(test_value))
        form_result = value.value
        assert 'count' in form_result
        assert form_result['count'] == 2
        # we dropped unknown data
        assert len(form_result) == 1


def test_default_value():
    test_value = {'testing': 1, 'count': 2}
    with form('TEST'):
        value = RuntimeSetting(key="TEST", raw_value=json.dumps(test_value))
        defalft_value = value.default_value
        assert defalft_value == {'amdefault': None}


def test_has_changed():
    test_value = {'testing': 1, 'count': 2}
    with form('TEST'):
        value = RuntimeSetting(key="TEST", raw_value=json.dumps(test_value))
        assert value.has_changed() is True


@pytest.mark.django_db
def test_delete_restores_default():
    test_value = {'testing': 1, 'count': 2}
    value = RuntimeSetting(key="TEST", raw_value=json.dumps(test_value))
    value.save()
    db_obj = value.__class__.objects.get(key='TEST')
    with form('TEST'):
        assert db_obj.value == {'count': 2}
        value.delete()
        db_obj2 = value.__class__.objects.get(key='TEST')
        assert db_obj2.raw_value == '{"amdefault": null}'
        assert db_obj2.value == {}


@pytest.mark.django_db
def test_runtimesettingswrapper():
    test_value = {'testing': 1, 'count': 2}
    test2_value = {'testing': 1, 'count': 4}
    RuntimeSetting.objects.create(key="TEST", raw_value=json.dumps(test_value))
    RuntimeSetting.objects.create(key="TEST2", raw_value=json.dumps(test2_value))

    class ListPerPageForm(Form):
        count = IntegerField(initial=25, min_value=1, max_value=99)

    registry.register('TEST', ListPerPageForm, {'count': '3'})
    registry.register('TEST_DEFAULT', ListPerPageForm, {'count': '14'})

    wrapped = RuntimeSettingWrapper()
    assert wrapped['TEST'] == {'count': 2}
    assert wrapped['TEST_DEFAULT'] == {'count': 14}

    # new values won't be discovered until the existing data is updated
    RuntimeSetting.objects.create(key="TEST_DEFAULT",
                                  raw_value=json.dumps(test2_value))
    assert wrapped['TEST_DEFAULT'] == {'count': 14}
    wrapped2 = RuntimeSettingWrapper()
    assert wrapped2['TEST_DEFAULT'] == {'count': 4}
    assert bool(wrapped) is True
    assert 'TEST' in wrapped
    assert wrapped.TEST == {'count': 2}
    data = set(x for x, v in wrapped)
    assert data == {'TEST', 'TEST_DEFAULT'}


@pytest.mark.django_db
def test_can_serialize_modelchoices():
    user1 = get_user_model().objects.create(username='woo')
    user2 = get_user_model().objects.create(username='woo2')
    with userform('USERFORM') as form_class:
        data = {'single_user': user1.pk,
                'many_users': [user1.pk, user2.pk],
                'another': 3}
        value = RuntimeSetting(key="USERFORM")
        value.value = data
        assert '"single_user": "1"' in value.raw_value
        assert '"many_users": ["1", "2"]' in value.raw_value
        assert value.value['single_user'] == user1
        assert set(value.value['many_users']) == set([user1, user2])
